/*
Copyright (c) 2014 Juergen Wuest

The MIT License

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Note: this license grant only applies to this Source File, and not to any
other part of SDMetrics or Velocity.

*/
package com.sdmetrics.velocity;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import com.sdmetrics.metrics.Metric;
import com.sdmetrics.metrics.MetricsEngine;
import com.sdmetrics.metrics.Rule;
import com.sdmetrics.metrics.RuleEngine;
import com.sdmetrics.metrics.RuleViolation;
import com.sdmetrics.metrics.Set;
import com.sdmetrics.model.MetaModel;
import com.sdmetrics.model.MetaModelElement;
import com.sdmetrics.model.Model;
import com.sdmetrics.model.ModelElement;

public class ReportFacade {

	private Model model;
	private MetaModel mm;
	private MetricsEngine me;
	private RuleEngine re;
	
	public ReportFacade(Model model) {
		this.model=model;
		this.mm=model.getMetaModel();
	}
	
	public ReportFacade(MetricsEngine me) {
		this(me.getModel());
		this.me=me;
	}
	
	public ReportFacade(RuleEngine re) {
		this(re.getMetricsEngine());
		this.re=re;
	}
	
	// Access to the metamodel
	
	public MetaModelElement getMetaModelElement(String name) {
		return mm.getType(name);
	}
	
	public List<MetaModelElement> getTypes() {
		List<MetaModelElement> typeList = new ArrayList<>();
		for(MetaModelElement mme : mm) {
			typeList.add(mme);
		}
		return typeList;
	}
	
	// Access to the model

	public List<ModelElement> getElements(MetaModelElement type) {
		return model.getAcceptedElements(type);
	}
	
	public List<ModelElement> getRootElements() {
		ArrayList<ModelElement> roots=new ArrayList<>();
		for(ModelElement me : model) {
			if(me.getOwner()==null) {
				roots.add(me);
			}
		}
		return roots;
	}
	
	public boolean isModelElement(Object o) {
		return o instanceof ModelElement;
	}
	
	// Access to metrics and sets (requires a metrics engine)
	
	public Collection<Metric> getMetrics(MetaModelElement type) {
		return me.getMetricStore().getMetrics(type);
	}
	
	public Object getMetricValue(ModelElement e, Metric m) {
		return me.getMetricValue(e, m);
	}
	
	public Collection<Set> getSets(MetaModelElement type) {
		return me.getMetricStore().getSets(type);
	}
	
	public Collection<?> getSet(ModelElement e, Set s) {
		return me.getSet(e, s);
	}
	
	// Access to rule violations (requires a rule engine)
	
	public Collection<Rule> getRules(MetaModelElement type) {
		return me.getMetricStore().getRules(type);
	}
	
	public List<RuleViolation> getRuleViolations(ModelElement e) {
		List<RuleViolation> violations=new ArrayList<>();
		Collection<Rule> rules=getRules(e.getType());
		for(Rule rule : rules) {
			violations.addAll(re.checkRule(e, rule));
		}
		
		return violations;
	}
}
