/*
Copyright (c) 2014 Juergen Wuest

The MIT License

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Note: this license grant only applies to this Source File, and not to any
other part of SDMetrics or Velocity.

*/
package com.sdmetrics.velocity;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

import com.sdmetrics.metrics.MetricStore;
import com.sdmetrics.metrics.MetricsEngine;
import com.sdmetrics.metrics.RuleEngine;
import com.sdmetrics.model.MetaModel;
import com.sdmetrics.model.Model;
import com.sdmetrics.model.XMIReader;
import com.sdmetrics.model.XMITransformations;

public class ReportLauncher
{
	static final String XMI_FILE = "sample.xmi";
	static final String TEMPLATE_FILE = "sdmetricstemplate.vm";
	static final String OUTPUT_FILE = "samplereport.html";
	static final String ENCODING = "UTF-8";
	
	public static void main(String[] args) throws Exception
	{
		
		// Parse the XMI file
		
		SAXParserFactory spf = SAXParserFactory.newInstance();
		spf.setValidating(false);
		spf.setNamespaceAware(false);
		SAXParser saxParser = spf.newSAXParser();

		String metaModelURL = MetaModel.class.getResource("/com/sdmetrics/resources/metamodel2.xml").toURI().toString(); 
		MetaModel metaModel = new MetaModel();
		saxParser.parse(metaModelURL, metaModel.getSAXParserHandler());

		String xmiTransURL = XMITransformations.class.getResource("/com/sdmetrics/resources/xmiTrans2_0.xml").toURI().toString();
		XMITransformations trans=new XMITransformations(metaModel);
		saxParser.parse(xmiTransURL, trans.getSAXParserHandler());

		Model model = new Model(metaModel);
		XMIReader xmiReader = new XMIReader(trans, model);
		saxParser.parse(XMI_FILE, xmiReader);

		// Instantiate metrics and rule engine
		
		String metricsURL = MetricStore.class.getResource("/com/sdmetrics/resources/metrics2.xml").toURI().toString();
		MetricStore metricStore = new MetricStore(metaModel);
		saxParser.parse(metricsURL, metricStore.getSAXParserHandler());
		
		MetricsEngine metricsEngine = new MetricsEngine(metricStore, model);
		RuleEngine ruleEngine = new RuleEngine(metricsEngine);
		
		// Generate the report
		
		Velocity.init( );
		VelocityContext context = new VelocityContext();
		context.put("report", new ReportFacade(ruleEngine));
		context.put("reportUtils", new ReportUtils());
		context.put("encoding", ENCODING);
		context.put("title", XMI_FILE);

		Template template = Velocity.getTemplate(TEMPLATE_FILE);

		BufferedWriter writer = new BufferedWriter(
				new OutputStreamWriter(new FileOutputStream(OUTPUT_FILE), ENCODING));

		template.merge(context, writer);

		writer.close();
		System.out.println("Report written to "+new File(OUTPUT_FILE).getAbsolutePath());
	}
}
